@@ -67,6 +67,9 @@ public class AvroWriteSupport<T> extends WriteSupport<T> {
   public static final String WRITE_PARQUET_UUID = "parquet.avro.write-parquet-uuid";
   static final boolean WRITE_PARQUET_UUID_DEFAULT = false;
 
+  // Support writing Parquet INT96 from a 12-byte Avro fixed.
+  public static final String WRITE_FIXED_AS_INT96 = "parquet.avro.writeFixedAsInt96";
+
   private static final String MAP_REPEATED_NAME = "key_value";
   private static final String MAP_KEY_NAME = "key";
   private static final String MAP_VALUE_NAME = "value";
@@ -124,7 +127,7 @@ public class AvroWriteSupport<T> extends WriteSupport<T> {
   public WriteContext init(Configuration configuration) {
     if (rootAvroSchema == null) {
       this.rootAvroSchema = new Schema.Parser().parse(configuration.get(AVRO_SCHEMA));
-      this.rootSchema = new AvroSchemaConverter().convert(rootAvroSchema);
+      this.rootSchema = new AvroSchemaConverter(configuration).convert(rootAvroSchema);
     }
 
     if (model == null) {
@@ -422,7 +425,7 @@ public class AvroWriteSupport<T> extends WriteSupport<T> {
       } else {
         Class<?> arrayClass = value.getClass();
         Preconditions.checkArgument(arrayClass.isArray(),
-            "Cannot write unless collection or array: " + arrayClass.getName());
+            "Cannot write unless collection or array: %s", arrayClass.getName());
         writeJavaArray(schema, avroSchema, arrayClass, value);
       }
       recordConsumer.endGroup();
@@ -440,7 +443,7 @@ public class AvroWriteSupport<T> extends WriteSupport<T> {
       switch (avroSchema.getElementType().getType()) {
         case BOOLEAN:
           Preconditions.checkArgument(elementClass == boolean.class,
-              "Cannot write as boolean array: " + arrayClass.getName());
+              "Cannot write as boolean array: %s", arrayClass.getName());
           writeBooleanArray((boolean[]) value);
           break;
         case INT:
@@ -459,17 +462,17 @@ public class AvroWriteSupport<T> extends WriteSupport<T> {
           break;
         case LONG:
           Preconditions.checkArgument(elementClass == long.class,
-              "Cannot write as long array: " + arrayClass.getName());
+              "Cannot write as long array: %s", arrayClass.getName());
           writeLongArray((long[]) value);
           break;
         case FLOAT:
           Preconditions.checkArgument(elementClass == float.class,
-              "Cannot write as float array: " + arrayClass.getName());
+              "Cannot write as float array: %s", arrayClass.getName());
           writeFloatArray((float[]) value);
           break;
         case DOUBLE:
           Preconditions.checkArgument(elementClass == double.class,
-              "Cannot write as double array: " + arrayClass.getName());
+              "Cannot write as double array: %s", arrayClass.getName());
           writeDoubleArray((double[]) value);
           break;
         default:
