@@ -255,7 +255,8 @@ public class TestReadWrite {
       Random random = new Random(34L);
       GenericRecordBuilder builder = new GenericRecordBuilder(decimalSchema);
       for (int i = 0; i < 1000; i += 1) {
-        BigDecimal dec = new BigDecimal(new BigInteger(31, random), 2);
+        // Generating Integers between -(2^29) and (2^29 - 1) to ensure the number of digits <= 9
+        BigDecimal dec = new BigDecimal(new BigInteger(30, random).subtract(BigInteger.valueOf(1L << 28)), 2);
         builder.set("dec", dec);
 
         GenericRecord rec = builder.build();
@@ -307,7 +308,8 @@ public class TestReadWrite {
       Random random = new Random(34L);
       GenericRecordBuilder builder = new GenericRecordBuilder(decimalSchema);
       for (int i = 0; i < 1000; i += 1) {
-        BigDecimal dec = new BigDecimal(new BigInteger(31, random), 2);
+        // Generating Integers between -(2^29) and (2^29 - 1) to ensure the number of digits <= 9
+        BigDecimal dec = new BigDecimal(new BigInteger(30, random).subtract(BigInteger.valueOf(1L << 28)), 2);
         builder.set("dec", dec);
 
         GenericRecord rec = builder.build();
@@ -727,6 +729,52 @@ public class TestReadWrite {
     assertEquals(str("hello"), l2List.get(0));
   }
 
+  /**
+   * A test demonstrating the most simple way to write and read Parquet files
+   * using Avro {@link GenericRecord}.
+   */
+  @Test
+  public void testSimpleGeneric() throws IOException {
+    final Schema schema =
+        Schema.createRecord("Person", null, "org.apache.parquet", false);
+    schema.setFields(Arrays.asList(
+        new Schema.Field("name", Schema.create(Schema.Type.STRING), null, null),
+        new Schema.Field("weight", Schema.create(Schema.Type.INT), null,
+            null)));
+
+    final Path file = new Path(createTempFile().getPath());
+
+    try (final ParquetWriter<GenericData.Record> parquetWriter =
+        AvroParquetWriter.<GenericData.Record> builder(file).withSchema(schema)
+            .build()) {
+
+      final GenericData.Record fooRecord = new GenericData.Record(schema);
+      fooRecord.put("name", "foo");
+      fooRecord.put("weight", 123);
+
+      final GenericData.Record oofRecord = new GenericData.Record(schema);
+      oofRecord.put("name", "oof");
+      oofRecord.put("weight", 321);
+
+      parquetWriter.write(fooRecord);
+      parquetWriter.write(oofRecord);
+    }
+
+    // Read the file. String data is returned as org.apache.avro.util.Utf8 so it
+    // must be converting to a String before checking equality
+    try (ParquetReader<GenericRecord> reader =
+        AvroParquetReader.genericRecordReader(file)) {
+
+      final GenericRecord r1 = reader.read();
+      assertEquals("foo", r1.get("name").toString());
+      assertEquals(123, r1.get("weight"));
+
+      final GenericRecord r2 = reader.read();
+      assertEquals("oof", r2.get("name").toString());
+      assertEquals(321, r2.get("weight"));
+    }
+  }
+
   private File createTempFile() throws IOException {
     File tmp = File.createTempFile(getClass().getSimpleName(), ".tmp");
     tmp.deleteOnExit();
