@@ -18,6 +18,8 @@
  */
 package org.apache.parquet.column.page;
 
+import java.util.OptionalInt;
+
 /**
  * one page in a chunk
  */
@@ -43,4 +45,18 @@ abstract public class Page {
     return uncompressedSize;
   }
 
+  // Note: the following field is only used for testing purposes and are NOT used in checksum
+  // verification. There crc value here will merely be a copy of the actual crc field read in
+  // ParquetFileReader.Chunk.readAllPages()
+  private OptionalInt crc = OptionalInt.empty();
+
+  // Visible for testing
+  public void setCrc(int crc) {
+    this.crc = OptionalInt.of(crc);
+  }
+
+  // Visible for testing
+  public OptionalInt getCrc() {
+    return crc;
+  }
 }
