@@ -26,11 +26,13 @@ import static org.apache.parquet.filter2.predicate.FilterApi.eq;
 import static org.apache.parquet.filter2.predicate.FilterApi.floatColumn;
 import static org.apache.parquet.filter2.predicate.FilterApi.gt;
 import static org.apache.parquet.filter2.predicate.FilterApi.gtEq;
+import static org.apache.parquet.filter2.predicate.FilterApi.in;
 import static org.apache.parquet.filter2.predicate.FilterApi.intColumn;
 import static org.apache.parquet.filter2.predicate.FilterApi.longColumn;
 import static org.apache.parquet.filter2.predicate.FilterApi.lt;
 import static org.apache.parquet.filter2.predicate.FilterApi.ltEq;
 import static org.apache.parquet.filter2.predicate.FilterApi.notEq;
+import static org.apache.parquet.filter2.predicate.FilterApi.notIn;
 import static org.apache.parquet.filter2.predicate.FilterApi.userDefined;
 import static org.apache.parquet.filter2.predicate.LogicalInverter.invert;
 import static org.apache.parquet.schema.OriginalType.DECIMAL;
@@ -55,7 +57,9 @@ import java.math.BigDecimal;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Comparator;
+import java.util.HashSet;
 import java.util.List;
+import java.util.Set;
 
 import org.apache.parquet.bytes.BytesUtils;
 import org.apache.parquet.column.statistics.Statistics;
@@ -274,6 +278,13 @@ public class TestColumnIndexBuilder {
         decimalBinary("87656273"));
     assertCorrectFiltering(columnIndex, eq(col, decimalBinary("0.0")), 1, 4);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 2, 3, 5, 6);
+    Set<Binary> set1 = new HashSet<>();
+    set1.add(Binary.fromString("0.0"));
+    assertCorrectFiltering(columnIndex, in(col, set1), 1, 4);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5, 6, 7);
+    set1.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set1), 0, 1, 2, 3, 4, 5, 6);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5, 6, 7);
     assertCorrectFiltering(columnIndex, notEq(col, decimalBinary("87656273")), 0, 1, 2, 3, 4, 5, 6);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 2, 4, 7);
     assertCorrectFiltering(columnIndex, gt(col, decimalBinary("2348978.45")), 1);
@@ -319,6 +330,13 @@ public class TestColumnIndexBuilder {
         null);
     assertCorrectFiltering(columnIndex, eq(col, decimalBinary("87656273")), 2, 4);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 3, 5, 6, 7);
+    Set<Binary> set2 = new HashSet<>();
+    set2.add(decimalBinary("87656273"));
+    assertCorrectFiltering(columnIndex, in(col, set2), 2, 4);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7);
+    set2.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set2), 0, 2, 3, 4, 5, 6, 7);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7);
     assertCorrectFiltering(columnIndex, notEq(col, decimalBinary("87656273")), 0, 1, 2, 3, 5, 6, 7);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 2, 4, 6);
     assertCorrectFiltering(columnIndex, gt(col, decimalBinary("87656273")), 6);
@@ -364,6 +382,13 @@ public class TestColumnIndexBuilder {
         decimalBinary("-9999293.23"));
     assertCorrectFiltering(columnIndex, eq(col, decimalBinary("1234567890.12")), 2, 4);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 1, 2, 3, 6);
+    Set<Binary> set3 = new HashSet<>();
+    set3.add(decimalBinary("1234567890.12"));
+    assertCorrectFiltering(columnIndex, in(col, set3), 2, 4);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7);
+    set3.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set3), 0, 1, 2, 3, 4, 6);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7);
     assertCorrectFiltering(columnIndex, notEq(col, decimalBinary("0.0")), 0, 1, 2, 3, 4, 5, 6, 7);
     assertCorrectFiltering(columnIndex, notEq(col, null), 2, 4, 5, 7);
     assertCorrectFiltering(columnIndex, gt(col, decimalBinary("1234567890.12")));
@@ -417,6 +442,13 @@ public class TestColumnIndexBuilder {
         null);
     assertCorrectFiltering(columnIndex, eq(col, stringBinary("Marvin")), 1, 4, 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 1, 2, 3, 5, 7);
+    Set<Binary> set1 = new HashSet<>();
+    set1.add(stringBinary("Marvin"));
+    assertCorrectFiltering(columnIndex, in(col, set1), 1, 4, 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5, 6, 7);
+    set1.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set1), 0, 1, 2, 3, 4, 5, 7);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5, 6, 7);
     assertCorrectFiltering(columnIndex, notEq(col, stringBinary("Beeblebrox")), 0, 1, 2, 3, 4, 5, 7);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 4, 5, 6);
     assertCorrectFiltering(columnIndex, gt(col, stringBinary("Prefect")), 1, 5);
@@ -462,6 +494,13 @@ public class TestColumnIndexBuilder {
         null);
     assertCorrectFiltering(columnIndex, eq(col, stringBinary("Jeltz")), 3, 4);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 1, 2, 4, 5, 7);
+    Set<Binary> set2 = new HashSet<>();
+    set2.add( stringBinary("Jeltz"));
+    assertCorrectFiltering(columnIndex, in(col, set2), 3, 4);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7);
+    set2.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set2), 0, 1, 2, 3, 4, 5, 7);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7);
     assertCorrectFiltering(columnIndex, notEq(col, stringBinary("Slartibartfast")), 0, 1, 2, 3, 4, 5, 7);
     assertCorrectFiltering(columnIndex, notEq(col, null), 0, 3, 4, 6);
     assertCorrectFiltering(columnIndex, gt(col, stringBinary("Marvin")), 4, 6);
@@ -507,6 +546,13 @@ public class TestColumnIndexBuilder {
         stringBinary("Beeblebrox"));
     assertCorrectFiltering(columnIndex, eq(col, stringBinary("Marvin")), 3);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 2, 3, 5, 6, 7);
+    Set<Binary> set3 = new HashSet<>();
+    set3.add(stringBinary("Marvin"));
+    assertCorrectFiltering(columnIndex, in(col, set3), 3);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7);
+    set3.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set3), 0, 2, 3, 5, 6, 7);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7);
     assertCorrectFiltering(columnIndex, notEq(col, stringBinary("Dent")), 0, 1, 2, 3, 5, 6, 7);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 3, 4, 7);
     assertCorrectFiltering(columnIndex, gt(col, stringBinary("Prefect")), 1);
@@ -615,6 +661,13 @@ public class TestColumnIndexBuilder {
     BinaryColumn col = binaryColumn("test_col");
     assertCorrectFiltering(columnIndex, eq(col, stringBinary("Dent")), 2, 3);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 1, 2, 3, 4, 5, 6, 7);
+    Set<Binary> set = new HashSet<>();
+    set.add(stringBinary("Dent"));
+    assertCorrectFiltering(columnIndex, in(col, set), 2, 3);
+    assertCorrectFiltering(columnIndex, notIn(col, set), 0, 1, 2, 3, 4, 5, 6, 7);
+    set.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set), 0, 1, 2, 3, 4, 5, 6, 7);
+    assertCorrectFiltering(columnIndex, notIn(col, set), 0, 1, 2, 3, 4, 5, 6, 7);
     assertCorrectFiltering(columnIndex, notEq(col, stringBinary("Dent")), 0, 1, 2, 3, 4, 5, 6, 7);
     assertCorrectFiltering(columnIndex, notEq(col, null), 2, 3, 5, 7);
     assertCorrectFiltering(columnIndex, userDefined(col, BinaryDecimalIsNullOrZeroUdp.class), 0, 1, 2, 3, 4, 5, 6, 7);
@@ -646,6 +699,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), false, false, true, null, false);
     assertCorrectFiltering(columnIndex, eq(col, true), 0, 1, 2);
     assertCorrectFiltering(columnIndex, eq(col, null), 1, 2, 3);
+    Set<Boolean> set1 = new HashSet<>();
+    set1.add(true);
+    assertCorrectFiltering(columnIndex, in(col, set1), 0, 1, 2);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4);
+    set1.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set1), 0, 1, 2, 3);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4);
     assertCorrectFiltering(columnIndex, notEq(col, true), 0, 1, 2, 3, 4);
     assertCorrectFiltering(columnIndex, notEq(col, null), 0, 1, 2, 4);
     assertCorrectFiltering(columnIndex, userDefined(col, BooleanIsTrueOrNull.class), 0, 1, 2, 3);
@@ -670,6 +730,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), null, false, null, null, false, false, null);
     assertCorrectFiltering(columnIndex, eq(col, true), 4, 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 2, 3, 4, 5, 6);
+    Set<Boolean> set2 = new HashSet<>();
+    set2.add(true);
+    assertCorrectFiltering(columnIndex, in(col, set2), 4, 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6);
+    set2.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set2), 0, 2, 3, 4, 5, 6);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6);
     assertCorrectFiltering(columnIndex, notEq(col, true), 0, 1, 2, 3, 4, 5, 6);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 4, 5);
     assertCorrectFiltering(columnIndex, userDefined(col, BooleanIsTrueOrNull.class), 0, 2, 3, 4, 5, 6);
@@ -694,6 +761,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), null, true, null, null, false, false, null);
     assertCorrectFiltering(columnIndex, eq(col, true), 1, 4);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 2, 3, 4, 5, 6);
+    Set<Boolean> set3 = new HashSet<>();
+    set3.add(true);
+    assertCorrectFiltering(columnIndex, in(col, set3), 1, 4);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6);
+    set3.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set3), 0, 1, 2, 3, 4, 5, 6);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6);
     assertCorrectFiltering(columnIndex, notEq(col, true), 0, 2, 3, 4, 5, 6);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 4, 5);
     assertCorrectFiltering(columnIndex, userDefined(col, BooleanIsTrueOrNull.class), 0, 1, 2, 3, 4, 5, 6);
@@ -741,6 +815,14 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), -4.2, -11.7, 2.2, null, 1.9, -21.0);
     assertCorrectFiltering(columnIndex, eq(col, 0.0), 1, 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 1, 2, 3);
+    Set<Double> set1 = new HashSet<>();
+    set1.add(0.0);
+    set1.add(-4.2);
+    assertCorrectFiltering(columnIndex, in(col, set1), 0, 1, 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5);
+    set1.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set1), 0, 1, 2, 3, 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5);
     assertCorrectFiltering(columnIndex, notEq(col, 2.2), 0, 1, 2, 3, 4, 5);
     assertCorrectFiltering(columnIndex, notEq(col, null), 0, 1, 2, 4, 5);
     assertCorrectFiltering(columnIndex, gt(col, 2.2), 1, 4, 5);
@@ -771,6 +853,15 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), null, -532.3, -234.7, null, null, -234.6, null, 3.0, null);
     assertCorrectFiltering(columnIndex, eq(col, 0.0), 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 1, 2, 3, 4, 6, 8);
+    Set<Double> set2 = new HashSet<>();
+    set2.add(0.0);
+    set2.add(3.5);
+    set2.add(-346.0);
+    assertCorrectFiltering(columnIndex, in(col, set2), 1, 2, 5, 7);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
+    set2.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, 0.0), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 2, 5, 7);
     assertCorrectFiltering(columnIndex, gt(col, 2.99999), 7);
@@ -801,6 +892,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), null, 345.2, null, 234.6, null, -2.99999, null, null, -42.83);
     assertCorrectFiltering(columnIndex, eq(col, 234.6), 3, 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 2, 3, 4, 6, 7);
+    Set<Double> set3 = new HashSet<>();
+    set3.add(234.6);
+    assertCorrectFiltering(columnIndex, in(col, set3), 3, 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7, 8);
+    set3.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set3), 0, 2, 3, 4, 5, 6, 7);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, 2.2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 3, 5, 8);
     assertCorrectFiltering(columnIndex, gt(col, 2.2), 1, 3, 5);
@@ -871,6 +969,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), -4.2f, -11.7f, 2.2f, null, 1.9f, -21.0f);
     assertCorrectFiltering(columnIndex, eq(col, 0.0f), 1, 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 1, 2, 3);
+    Set<Float> set1 = new HashSet<>();
+    set1.add(0.0f);
+    assertCorrectFiltering(columnIndex, in(col, set1), 1, 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5);
+    set1.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set1), 1, 2, 3, 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5);
     assertCorrectFiltering(columnIndex, notEq(col, 2.2f), 0, 1, 2, 3, 4, 5);
     assertCorrectFiltering(columnIndex, notEq(col, null), 0, 1, 2, 4, 5);
     assertCorrectFiltering(columnIndex, gt(col, 2.2f), 1, 4, 5);
@@ -901,6 +1006,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), null, -532.3f, -300.6f, null, null, -234.6f, null, 3.0f, null);
     assertCorrectFiltering(columnIndex, eq(col, 0.0f), 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 1, 2, 3, 4, 6, 8);
+    Set<Float> set2 = new HashSet<>();
+    set2.add(0.0f);
+    assertCorrectFiltering(columnIndex, in(col, set2), 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
+    set2.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set2), 0, 1, 2, 3, 4, 5, 6, 8);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, 2.2f), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 2, 5, 7);
     assertCorrectFiltering(columnIndex, gt(col, 2.2f), 5, 7);
@@ -931,6 +1043,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), null, 345.2f, null, 234.6f, null, -2.99999f, null, null, -42.83f);
     assertCorrectFiltering(columnIndex, eq(col, 234.65f), 3);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 2, 3, 4, 6, 7);
+    Set<Float> set3 = new HashSet<>();
+    set3.add(234.65f);
+    assertCorrectFiltering(columnIndex, in(col, set3), 3);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7, 8);
+    set3.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set3), 0, 2, 3, 4, 6, 7);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, 2.2f), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 3, 5, 8);
     assertCorrectFiltering(columnIndex, gt(col, 2.2f), 1, 3, 5);
@@ -1001,6 +1120,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), -4, -11, 2, null, 1, -21);
     assertCorrectFiltering(columnIndex, eq(col, 2), 0, 1, 2, 4, 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 1, 2, 3);
+    Set<Integer> set1 = new HashSet<>();
+    set1.add(2);
+    assertCorrectFiltering(columnIndex, in(col, set1), 0, 1, 2, 4, 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5);
+    set1.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set1), 0, 1, 2, 3, 4, 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5);
     assertCorrectFiltering(columnIndex, notEq(col, 2), 0, 1, 2, 3, 4, 5);
     assertCorrectFiltering(columnIndex, notEq(col, null), 0, 1, 2, 4, 5);
     assertCorrectFiltering(columnIndex, gt(col, 2), 0, 1, 5);
@@ -1031,6 +1157,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), null, -532, -500, null, null, -42, null, 3, null);
     assertCorrectFiltering(columnIndex, eq(col, 2), 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 1, 2, 3, 4, 6, 8);
+    Set<Integer> set2 = new HashSet<>();
+    set2.add(2);
+    assertCorrectFiltering(columnIndex, in(col, set2), 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
+    set2.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set2), 0, 1, 2, 3, 4, 5, 6, 8);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, 2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 2, 5, 7);
     assertCorrectFiltering(columnIndex, gt(col, 2), 7);
@@ -1062,6 +1195,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), null, 345, null, 42, null, -2, null, null, -42);
     assertCorrectFiltering(columnIndex, eq(col, 2), 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 2, 3, 4, 6, 7);
+    Set<Integer> set3 = new HashSet<>();
+    set3.add(2);
+    assertCorrectFiltering(columnIndex, in(col, set3), 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7, 8);
+    set3.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set3), 0, 2, 3, 4, 5, 6, 7);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, 2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 3, 5, 8);
     assertCorrectFiltering(columnIndex, gt(col, 2), 1, 3, 5);
@@ -1114,6 +1254,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), 4, 11, 2, null, 1, 0xEF);
     assertCorrectFiltering(columnIndex, eq(col, 2), 2, 4);
     assertCorrectFiltering(columnIndex, eq(col, null), 1, 2, 3);
+    Set<Integer> set1 = new HashSet<>();
+    set1.add(2);
+    assertCorrectFiltering(columnIndex, in(col, set1), 2, 4);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5);
+    set1.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set1), 1, 2, 3, 4);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5);
     assertCorrectFiltering(columnIndex, notEq(col, 2), 0, 1, 2, 3, 4, 5);
     assertCorrectFiltering(columnIndex, notEq(col, null), 0, 1, 2, 4, 5);
     assertCorrectFiltering(columnIndex, gt(col, 2), 0, 1, 4, 5);
@@ -1144,6 +1291,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), null, 0, 0, null, null, 42, null, 0xEF, null);
     assertCorrectFiltering(columnIndex, eq(col, 2), 2);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 1, 2, 3, 4, 6, 8);
+    Set<Integer> set2 = new HashSet<>();
+    set2.add(2);
+    assertCorrectFiltering(columnIndex, in(col, set2), 2);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
+    set2.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set2), 0, 1, 2, 3, 4, 6, 8);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, 2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 2, 5, 7);
     assertCorrectFiltering(columnIndex, gt(col, 0xEE), 7);
@@ -1175,6 +1329,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), null, 0xFF, null, 0xEA, null, 42, null, null, 0);
     assertCorrectFiltering(columnIndex, eq(col, 0xAB), 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 2, 3, 4, 6, 7);
+    Set<Integer> set3 = new HashSet<>();
+    set3.add(0xAB);
+    assertCorrectFiltering(columnIndex, in(col, set3), 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7, 8);
+    set3.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set3), 0, 2, 3, 4, 5, 6, 7);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, 0xFF), 0, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 3, 5, 8);
     assertCorrectFiltering(columnIndex, gt(col, 0xFF));
@@ -1211,6 +1372,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), -4l, -11l, 2l, null, 1l, -21l);
     assertCorrectFiltering(columnIndex, eq(col, 0l), 0, 1, 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 1, 2, 3);
+    Set<Long> set1 = new HashSet<>();
+    set1.add(0l);
+    assertCorrectFiltering(columnIndex, in(col, set1), 0, 1, 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5);
+    set1.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set1), 0, 1, 2, 3, 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set1), 0, 1, 2, 3, 4, 5);
     assertCorrectFiltering(columnIndex, notEq(col, 0l), 0, 1, 2, 3, 4, 5);
     assertCorrectFiltering(columnIndex, notEq(col, null), 0, 1, 2, 4, 5);
     assertCorrectFiltering(columnIndex, gt(col, 2l), 0, 1, 5);
@@ -1241,6 +1409,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), null, -532l, -234l, null, null, -42l, null, -3l, null);
     assertCorrectFiltering(columnIndex, eq(col, -42l), 2, 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 1, 2, 3, 4, 6, 8);
+    Set<Long> set2 = new HashSet<>();
+    set2.add(-42l);
+    assertCorrectFiltering(columnIndex, in(col, set2), 2, 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
+    set2.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set2), 0, 1, 2, 3, 4, 5, 6, 8);
+    assertCorrectFiltering(columnIndex, notIn(col, set2), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, -42l), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 2, 5, 7);
     assertCorrectFiltering(columnIndex, gt(col, 2l), 7);
@@ -1272,6 +1447,13 @@ public class TestColumnIndexBuilder {
     assertCorrectValues(columnIndex.getMinValues(), null, 345l, null, 42l, null, -2l, null, null, -42l);
     assertCorrectFiltering(columnIndex, eq(col, 0l), 5);
     assertCorrectFiltering(columnIndex, eq(col, null), 0, 2, 3, 4, 6, 7);
+    Set<Long> set3 = new HashSet<>();
+    set3.add(0l);
+    assertCorrectFiltering(columnIndex, in(col, set3), 5);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7, 8);
+    set3.add(null);
+    assertCorrectFiltering(columnIndex, in(col, set3), 0, 2, 3, 4, 5, 6, 7);
+    assertCorrectFiltering(columnIndex, notIn(col, set3), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, 0l), 0, 1, 2, 3, 4, 5, 6, 7, 8);
     assertCorrectFiltering(columnIndex, notEq(col, null), 1, 3, 5, 8);
     assertCorrectFiltering(columnIndex, gt(col, 2l), 1, 3, 5);
